/* Climb evaluator
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.util.HashSet;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.test.FBTestGenerator;
import com.bbn.morph.view.AttributeValueView;
import com.bbn.morph.view.RobotAttribute;

public class ClimbEvaluator implements Evaluator {

  private AttributeValueView m_model;
  private String m_rFlip;
  private String m_lFlip;
  private String m_body;
  private String m_featureName;
  private String m_stressType;

  public ClimbEvaluator(AttributeValueView model, String featureName, 
      String stressType, String rightFlipper,
      String leftFlipper, String body) {
    m_model = model;
    m_featureName = featureName;
    m_stressType = stressType;
    m_rFlip = rightFlipper;
    m_lFlip = leftFlipper;
    m_body = body;
  }
  
  @Deprecated
  public ClimbEvaluator(AttributeValueView model) {
    this(model, FBTestGenerator.boxFeature, StressInputType.FLIPPER_BODY_RATIO.toString().toLowerCase(),
        FBTestGenerator.rightflipperLengthAttribute,
        FBTestGenerator.leftflipperLengthAttribute,
        FBTestGenerator.bodyLengthAttribute);
  }
  
  @Override
  public Set<StressInputs> getStressInputs() {
	  m_model.refresh();
    Measurement flipperLength = Measurement.min(m_model.get(new RobotAttribute(m_rFlip)),
    		                        m_model.get(new RobotAttribute(m_lFlip)));
    Measurement bodyLength = m_model.get(new RobotAttribute(m_body));
    
    Measurement totalLength = flipperLength.add(bodyLength);
    
    // 4 is max stress, 6 is medium, 8 is not stressed
    double minStress = 4;
    double maxStress = 7.75;
    
    double ratio = 1.0 - (totalLength.getStandardValue() - minStress) / (maxStress - minStress);
    ratio = Math.max(0.0, ratio);
    ratio = Math.min(1.0, ratio);
    // ratio between 0 and 1
    
    // start at 3000 increase by up to 2000
    double time = 3000 + ratio * 2000;

    StressInputs si = new StressInputs(new Feature(m_featureName));
    si.put(StressInputType.fromString(m_stressType), time);
    
    HashSet<StressInputs> results = new HashSet<StressInputs>();
    results.add(si);
    return results;
  }
}
