/* Evaluator for comparing current values
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.test.FBTestGenerator;
import com.bbn.morph.view.AttributeValueView;
import com.bbn.morph.view.RobotAttribute;

public class CurrentValueEvaluator implements Evaluator {

  private AttributeValueView m_model;
  private List<String> m_stressTypes;
  private List<String> m_stressFeatures;
  private List<String> m_stressInputs;

  @Deprecated
  public CurrentValueEvaluator(AttributeValueView model) {
    this(model, 
        Arrays.asList(StressInputType.TORQUE.toString(), StressInputType.MOTOR_MASS.toString()),
        Arrays.asList(FBTestGenerator.torqueAttribute, FBTestGenerator.motorMassAttribute),
        Arrays.asList(FBTestGenerator.MaxTorqueFeature, FBTestGenerator.MotorMassFeature));
  }
    
  public CurrentValueEvaluator(AttributeValueView model, 
      List<String> stressTypes, List<String> stressFeatures, 
      List<String> stressInputs) {
    m_model = model;
    m_stressFeatures = stressFeatures;
    m_stressTypes = stressTypes;
    m_stressInputs = stressInputs;
  }

  
  @Override
  public Set<StressInputs> getStressInputs() {
    m_model.refresh();  
    
    HashSet<StressInputs> results = new HashSet<StressInputs>();
    
    int values = Math.min(Math.min(m_stressTypes.size(), m_stressFeatures.size()), 
        m_stressInputs.size());
    
    for (int i=0; i<values; i++) {
      Measurement m = m_model.get(new RobotAttribute(m_stressFeatures.get(i)));
      StressInputs si = new StressInputs(new Feature(m_stressInputs.get(i)));
      si.put(StressInputType.fromString(m_stressTypes.get(i)), m.getStandardValue()); 
      results.add(si);
    }

    return results;
  }

}
