/* Ratio evaluation
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.view.AttributeValueView;
import com.bbn.morph.view.RobotAttribute;

public class RatioEvaluator implements Evaluator {

  private AttributeValueView m_model;
  Map<Feature,List<String>> ratios;

  public RatioEvaluator(AttributeValueView model,Set<Feature> features){
    m_model = model;
    ratios = new HashMap<Feature,List<String>>();
    for(Feature f: features){
      String str =f.getName();
      String[] components = str.split("_");
      if (!components[0].equalsIgnoreCase("ratio"))
        continue;
      List<String> pair= new ArrayList<String>(2);
      pair.add(components[1]);
      pair.add(components[2]);
      ratios.put(f,pair);  	
    }

  }

  @Override
  public Set<StressInputs> getStressInputs() {
    m_model.refresh();  
    HashSet<StressInputs> results = new HashSet<StressInputs>();	
    for (Feature f: ratios.keySet()){
      List<String> pair = ratios.get(f);
      Measurement v1 = m_model.get(new RobotAttribute(pair.get(0)));
      Measurement v2 = m_model.get(new RobotAttribute(pair.get(1)));
      double ratio = v1.getStandardValue() / v2.getStandardValue();
      StressInputs si = new StressInputs(f);
      si.put(StressInputType.RATIO, ratio);
      results.add(si);
    }
    return results;
  }


}
