/* Evaluator that connects to XMLRPC for simulation
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.net.URL;
import java.util.HashSet;
import java.util.Set;

import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcClientConfigImpl;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.CompositeModel;
import com.bbn.morph.model.test.FBTestGenerator;
import com.bbn.morph.view.ROSModelView;

public class XMLRPCEvaluator implements Evaluator {

  private CompositeModel m_model;
  private String m_featureName;
  private String m_stressType;

  @Deprecated
  public XMLRPCEvaluator(CompositeModel model) {
    this(model, FBTestGenerator.climbFeature, StressInputType.ANGLE.toString().toLowerCase());
  }

  public XMLRPCEvaluator(CompositeModel model, String featureName,
      String stressType) {
    m_model = model;
    m_featureName = featureName;
    m_stressType = stressType;
  }

  static String url = "http://127.0.0.1:7080/xmlrpc";

  @Override
  public Set<StressInputs> getStressInputs() {
    ROSModelView rmv = new ROSModelView(m_model.getRobotModel(), 
        m_model.getEnvironmentModel());
    String[] fileStrings = rmv.getFiles();


    HashSet<StressInputs> results = new HashSet<StressInputs>();
    Double angle=null;
    try {
      XmlRpcClientConfigImpl config = new XmlRpcClientConfigImpl();
      config.setServerURL(new URL(url));
      XmlRpcClient client = new XmlRpcClient();
      client.setConfig(config);
      //Object[] params = new Object[]{new String("madv")};
      angle = (Double) client.execute("request2", fileStrings);
      //TODO make exceptions more specific
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("XMLRPC evaluator terminated with exception.");	 
    }
    if (angle==null)
      return results;
    StressInputs si = new StressInputs(new Feature(m_featureName));
    si.put(StressInputType.fromString(m_stressType), angle);
    results.add(si);
    return results;
  }

//  public static void main( String args[] ) throws Exception {
//    CompositeModel model = ModelGenerator.getCompositeModel();
//    XMLRPCEvaluator eval = new XMLRPCEvaluator( model );
//    System.out.println(eval.getStressInputs().toString() );
//  }	
}
