/** Functional Blueprint Model
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. 
**/

package com.bbn.morph.executer;

import java.util.HashMap;
import java.util.Map;

import com.bbn.morph.evaluator.RobotDifferenceEvaluator;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.PhysicalModel;
import com.bbn.morph.view.FB;
import com.bbn.morph.view.RelativeIncrementDecrementUpdate;
import com.bbn.morph.view.RobotAttribute;
import com.bbn.morph.view.StressFunction;
import com.bbn.morph.view.SymmetricFivePieceLinearStressFunction;
import com.bbn.morph.view.UpdateFunction;


public class FBModel extends HashMap<Feature, FB>{

  private static final long serialVersionUID = 1L;
  
public static double updateRate = 0.1;
	public FBModel() {
		super();
	}

	
	/**
	 * This will generate a dynamic FB to support the user requested changes
	 * Previous FB for changing the same attribute   will be overwritten
	 * @param a the robot attribute
	 * @param finalValue the desired final value for the parameter
	 * @param initialValue the initial value for the parameter
	 * @param model the model that will be updated
	 */
	public void addFB(RobotAttribute a, Measurement finalValue, Measurement initialValue, PhysicalModel model) {
		double r0= 0; // or maybe something beyond 0 like 0.005;
		double r1= 2*Math.abs(finalValue.getStandardValue() - initialValue.getStandardValue())/initialValue.getStandardValue();
		StressFunction stressF = new SymmetricFivePieceLinearStressFunction(StressInputType.NORMALIZED_DIFF, 0, r0, r1);
		Map<RobotAttribute, UpdateFunction> updateFunctions= new HashMap<RobotAttribute, UpdateFunction>();
		boolean reverseRelationShip = true;
		UpdateFunction update = new RelativeIncrementDecrementUpdate(updateRate, model, a, finalValue,reverseRelationShip);
		updateFunctions.put(a,update);
		Feature feature = RobotDifferenceEvaluator.genFeature(a.getName());
		FB newFB = new FB(feature, updateFunctions, stressF);
		newFB.setUserInduced(true); // flag that this is a user induced FB
		put(feature,newFB);	
	}
	
	/**
	 * Removes a FB linked to the specified {@link RobotAttribute}
	 * @param a the robot attribute to use to look up and remove the FB
	 */
	public void removeFB(RobotAttribute a) {
	  remove(RobotDifferenceEvaluator.genFeature(a.getName()));
	}
	
	public void addFB(FB fb){
		put(fb.getFeature(),fb);
	}
}
