/* Test component library loading
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.model.test;

import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import com.bbn.morph.model.ComponentInstance;
import com.bbn.morph.model.ComponentLibraries;
import com.bbn.morph.model.ComponentLibrary;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.ModelBody;
import com.bbn.morph.model.ModelBox;
import com.bbn.morph.model.ModelPosition;
import com.bbn.morph.model.RobotAttributeValues;
import com.bbn.morph.model.Units;


public class ComponentLibrariesTest {

  @Test
  public void testComponentLibraries() {
    ComponentLibraries componentLibraries = new ComponentLibraries();
    componentLibraries.loadLibraries();
    ComponentLibrary flippers = componentLibraries.getLibrary("flipper");
    Measurement x = new Measurement(Units.getMeters(), 4.0);
    Measurement y = new Measurement(Units.getMeters(), 3.0);
    Measurement z = new Measurement(Units.getMeters(), 2.0);
    Measurement lx = new Measurement(Units.getMeters(), 4.0);
    Measurement ly = new Measurement(Units.getMeters(), 0.2);
    Measurement lz = new Measurement(Units.getMeters(), 0.2);
    Measurement mass = new Measurement(Units.getKilograms(), 1);
    
    ModelBody body = new ModelBox("test", "", new ModelPosition(x, y, z), null, lx, ly, lz, mass);
    List<ComponentInstance> componentList = flippers.getComponents(body);
    Assert.assertEquals(1, componentList.size());
    ComponentInstance ci = componentList.get(0);
    Assert.assertEquals("flipper3", ci.getName());
    
    List<RobotAttributeValues> attributes = ci.getAttributes();
    RobotAttributeValues rav = attributes.get(0);
    Assert.assertEquals("lengthz",rav.getAttribute());
    Assert.assertEquals(3, rav.getMeasurement().getStandardValue(),0);

    rav = attributes.get(1);
    Assert.assertEquals("mass",rav.getAttribute());
    Assert.assertEquals(1, rav.getMeasurement().getStandardValue(),0);
  }
  
//  @Test
//  public void testBreakUnits() {
//    Measurement one = new Measurement(Units.getMeters(), 3.0);
//    Measurement two = new Measurement(Units.getMeters(), 5.0);
//    Assert.assertEquals("meters", two.getUnits().getfullName());
//  }

}
