/* Tests of the Measurement class
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.model.test;

import org.junit.Assert;
import org.junit.Test;

import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.UnitPrefixes;
import com.bbn.morph.model.Units;


public class MeasurementTest {

	@Test
	public void testConstructor() {
		Measurement m = new Measurement(Units.getKilograms(), 5);
		Assert.assertEquals(5, m.getPrefixValue(), 0.0);
		Assert.assertEquals(5, m.getStandardValue(), 0.0);
		Assert.assertEquals(5000.0, m.getValueIn(UnitPrefixes.NONE).getPrefixValue(), 0.0);
		
	}
	
	@Test
	public void testKilo() {
	  Measurement m = new Measurement(Units.getKilograms(), 5);
	  Measurement m2 = m.copy();
	  Assert.assertEquals(m.getUnits().toString(), m2.getUnits().toString());
	  Assert.assertEquals(m.getStandardValue(), m2.getStandardValue(), 0);
    Assert.assertEquals(5, m.getStandardValue(), 0);
    
    m = new Measurement(Units.getGrams(), 5);
    m2 = m.copy();
    Assert.assertEquals(m.getUnits().toString(), m2.getUnits().toString());
    Assert.assertEquals(m.getStandardValue(), m2.getStandardValue(), 0);
    Assert.assertEquals(0.005, m.getStandardValue(), 0);
	}
	
  @Test
  public void testGrams() {
    Measurement m = new Measurement(Units.getGrams(), 5);
    Measurement m2 = m.copy();
    Assert.assertEquals(m.getUnits().toString(), m2.getUnits().toString());
    Assert.assertEquals(m.getStandardValue(), m2.getStandardValue(), 0);
    
    m = new Measurement(Units.getGrams(), 5);
    m2 = m.copy();
    Assert.assertEquals(m.getUnits().toString(), m2.getUnits().toString());
    Assert.assertEquals(m.getStandardValue(), m2.getStandardValue(), 0);
  }
  
  @Test
  public void testKilo2() {
    Measurement m = new Measurement(Units.getKilograms(), 5);
    m = m.addMD(2);
    
    Assert.assertEquals(Units.getKilograms().toString(), m.getUnits().toString());
    Assert.assertEquals(7, m.getStandardValue(), 0);
  }
  
  @Test
  public void testKilo3() {
    Measurement m = new Measurement(Units.getGrams(), 5);
    m = m.addMD(2);
    
    Assert.assertEquals(Units.getGrams().toString(), m.getUnits().toString());
    Assert.assertEquals(2.005, m.getStandardValue(), 0);
    Assert.assertEquals(2005, m.getPrefixValue(), 0);
  }
  
	// FIXME not at all convinced that the kilograms/grams operations are okay because we store in grams, but standard units will convert to kilograms... 
	// 
}
