/* High-level MADV User Interface
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.ui;

import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.border.Border;

import com.bbn.morph.controller.Controller;
import com.bbn.morph.controller.DiscreteUpdateController;
import com.bbn.morph.model.CompositeModel;
import com.bbn.morph.model.InvalidModelException;
import com.bbn.morph.model.ModelGenerator;

public class MadvUI implements InformationReporter {
	
	private JPanel top;
	private InfoPanel bottom;
	private BotPanel topLeft;
	private JPanel topRight;
  private StressChart m_stressChart;
  private JSplitPane m_splitPane1;
  private double m_p1Width;
  private JSplitPane m_splitPane2;
  private double m_p2Height;
  private JSplitPane m_splitPane3;
  private double m_p3Height;
  private JFrame m_frame;
  
  public MadvUI(CompositeModel compositeModel, Controller controller,
      DiscreteUpdateController discreteController) {
    createAndShowGUI(compositeModel, controller, discreteController);
  }
  
  // TODO Either extend JFrame or figure out what to do about setVisible
  private void createAndShowGUI(CompositeModel compositeModel, Controller controller,
      DiscreteUpdateController discreteController) {
    // When using a GLCanvas, we have to set the Popup-Menus to be HeavyWeight,   
	// so they display properly on top of the GLCanvas
    JPopupMenu.setDefaultLightWeightPopupEnabled(false);

    JFrame.setDefaultLookAndFeelDecorated(true);
    m_frame = new JFrame("MADV UI");
    m_frame.setLayout(new BoxLayout(m_frame.getContentPane(), BoxLayout.Y_AXIS));

    m_frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

    top = new JPanel();
    m_stressChart = new StressChart("Stresses");
    JPanel stressPanel = m_stressChart.createDemoPanel();
    stressPanel.setPreferredSize(new Dimension(100, 30));
    bottom = new InfoPanel();
    topLeft = new BotPanel(compositeModel);
    topRight = new ParameterPanel(compositeModel, controller, discreteController);
    top.setLayout(new BoxLayout(top, BoxLayout.X_AXIS));

    m_frame.addWindowListener(new WindowAdapter() {

      @Override
      public void windowClosing(WindowEvent e) {
        // Run this on another thread than the AWT event queue to
        // make sure the call to Animator.stop() completes before
        // exiting

        topLeft.stop();               
      }
    });
    topLeft.start();

    //top.setOpaque(true);
   // bottom.setOpaque(true);
   // topLeft.setOpaque(true);
   // topRight.setOpaque(true);
    //topLeft.setBackground(Color.RED);
    //topRight.setBackground(Color.BLUE);
    //bottom.setBackground(Color.GREEN);
    Border inner = BorderFactory.createEtchedBorder();
    Border outer = BorderFactory.createRaisedBevelBorder();
    Border combined = BorderFactory.createCompoundBorder(outer, inner);
    
    m_splitPane1 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false, topLeft, topRight);
    m_splitPane1.setOneTouchExpandable(true);
    m_p1Width = 0.575;
    
    m_splitPane2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false, m_splitPane1, stressPanel);
    m_p2Height = 0.6;
    
    m_splitPane3 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false, m_splitPane2, bottom);
    m_p3Height = 1;
    
    //stressPanel.setPreferredSize(new Dimension(300, 200));
    //bottom.setSize(new Dimension(257, 10));
    
    
    //topLeft.setBorder(combined);
    //topRight.setBorder(combined);
    bottom.setBorder(combined);

    m_frame.add(top);
    //frame.add(bottom);
    //top.add(topLeft);
    //top.add(topRight);
    top.add(m_splitPane3);
    m_frame.pack();

    // The other bits and pieces that make our program a bit more stable.
    m_frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    m_frame.setSize(800, 500);
    
    // try for full screen mode:
//    GraphicsDevice defaultScreenDevice = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
//    if (defaultScreenDevice.isFullScreenSupported())
//      defaultScreenDevice.setFullScreenWindow(m_frame);
    
//    topLeft.addComponentListener(new ComponentAdapter() {
//      @Override
//      public void componentResized(ComponentEvent e) {
//        m_p1Width = (double)topLeft.getWidth() / (double)m_frame.getWidth();
//      }
//    });
//    
//    m_splitPane1.addComponentListener(new ComponentAdapter() {
//      @Override
//      public void componentResized(ComponentEvent e) {
//        m_p2Height = (double)m_splitPane1.getHeight() / (double)m_frame.getHeight();
//      }
//    });
//    
//    m_splitPane2.addComponentListener(new ComponentAdapter() {
//      @Override
//      public void componentResized(ComponentEvent e) {
//        m_p3Height = (double)m_splitPane2.getHeight() / (double)m_frame.getHeight();
//      }
//    });
//    
//    m_frame.addComponentListener(new ComponentAdapter() {
//      @Override
//      public void componentResized(ComponentEvent e) {
//        System.out.println("foo");
//        setDividers();
//      }
//    });
    
    setDividers();
    
    m_frame.setVisible(true);
  }
  
  private void setDividers() {
    int width = m_frame.getWidth(); // 800
    int height = m_frame.getHeight(); //500
    
    m_splitPane1.setDividerLocation((int)Math.floor(width*m_p1Width));   // horiz  
    m_splitPane2.setDividerLocation((int)Math.floor(height*m_p2Height)); // vert
    m_splitPane3.setDividerLocation((int)Math.floor(height*m_p3Height)); // vert
  }

  public static void main(String[] args) {
    //Schedule a job for the event-dispatching thread:
    //creating and showing this application's GUI.
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        try {
          CompositeModel compositeModel = ModelGenerator.getCompositeModel();
          Controller controller = new Controller(compositeModel);
          DiscreteUpdateController discreteController = new DiscreteUpdateController(compositeModel);
          MadvUI madvUI = new MadvUI(compositeModel, controller, discreteController);
          discreteController.setInformationReporter(madvUI);
          controller.setInformationReporter(madvUI);
        }
        catch (InvalidModelException e) {
          e.printStackTrace();
        }
      }
    });
  }

	@Override
	public void reportInformation(String information) {
		bottom.printText(information);
	}
	
	@Override
	public void reportStress(int interation, String id, double value) {
	  m_stressChart.addStress(interation, id, value);
	}
}
