/* The robot model display
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.ui;

import javax.media.opengl.GL;
import javax.media.opengl.GL2;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;

import com.bbn.morph.model.DesiredPhysicalModel;
import com.bbn.morph.model.PhysicalModel;
import com.bbn.morph.view.UIDesiredView;
import com.bbn.morph.view.UIPhysicalView;
import com.jogamp.opengl.util.gl2.GLUT;

public class UIRobotDraw implements GLEventListener {

	private GLU glu;
	private GLUT glut;
	private int m_width;
	private int m_height;
	//private float angle = 0.0f;
	private UIPhysicalView m_bot;
	private UIPhysicalView m_env;
	private UIDesiredView m_desBot;
	private UIDesiredView m_desEnv;
	private boolean m_isEnvOn;
	private boolean m_isDesOn;
	private int m_xShift = 0;
	private int m_yShift = 0;
	private MouseTracker m_mouseTracker;
	  
	public UIRobotDraw(MouseTracker mouseTracker, PhysicalModel mdl, 
			PhysicalModel env, DesiredPhysicalModel dmdl, DesiredPhysicalModel denv) {
		m_mouseTracker = mouseTracker;
		m_bot = new UIPhysicalView(mdl);
		m_desBot = new UIDesiredView(mdl, dmdl);
		m_env = new UIPhysicalView(env);
		m_desEnv = new UIDesiredView(env, denv);    

		m_isEnvOn = false;
	}
	
	private void loadTrackballTransformation(GL2 gl) {
	    double[][] Tmp = m_mouseTracker.getCurrentRotation();
	    int glindex = 0;
	    double[] glmatrix = new double[16];

	    gl.glLoadIdentity();

	    for (int col = 0; col < 4; ++col) {
	      for (int row = 0; row < 4; ++row) {
	        glmatrix[glindex++] = Tmp[row][col];
	      }
	    }
	    gl.glLoadMatrixd(glmatrix, 0);

	  }

	  // Loads the non-rotational parts of the trackball positioning                                                 
	  // Added by jsmb 5/13/06
	  private void loadTrackballZoomTranslate(GL2 gl) {
	    double zoomFactor = m_mouseTracker.getZoomFactor();
	    double[] center = m_mouseTracker.getCenter();

	    gl.glScaled(zoomFactor, zoomFactor, zoomFactor);
	    gl.glTranslated(center[0], center[1], 0.0 );
	  }
	
	  @Override
	  public void display(GLAutoDrawable drawable) {
	    GL2 gl = drawable.getGL().getGL2();

	    m_width = drawable.getWidth();
	    m_height = drawable.getHeight();
	    m_mouseTracker.setWidthHeight(m_width, m_height);

	    gl.glClear(GL.GL_COLOR_BUFFER_BIT|GL.GL_DEPTH_BUFFER_BIT);			// Clear the colour and depth buffer

	    gl.glViewport(0, 0, m_width, m_height);											// Reset The Current Viewport

	    gl.glMatrixMode(GL2.GL_PROJECTION);									// Select The Projection Matrix
	    gl.glLoadIdentity();												// Reset The Projection Matrix

	    glu.gluPerspective(45.0f,(float)m_width/(float)m_height,1f,5000.0f);			// Calculate The Aspect Ratio Of The Window
	    gl.glTranslated(0, 0, -10);

	    gl.glMatrixMode(GL2.GL_MODELVIEW);									// Select The Modelview Matrix
	    //gl.glLoadIdentity();												// Reset The Modelview Matrix     
	    loadTrackballTransformation(gl);
	    loadTrackballZoomTranslate(gl);
	    
	    drawScene(drawable);
	  }

	@Override
	public void dispose(GLAutoDrawable arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	  public void init(GLAutoDrawable drawable) {
	    m_width = drawable.getWidth();
	    m_height = drawable.getHeight();

	    GL2 gl = drawable.getGL().getGL2();
	    glu = new GLU();
	    glut = new GLUT();

	    // TODO -- compare to proto visualizer.cpp constructor
	    gl.glShadeModel(GL2.GL_SMOOTH);
	    gl.glClearColor(0.0f,0.0f,0.0f,0.0f);
	    gl.glClearDepth(1.0f);												// Depth Buffer Setup
	    gl.glEnable(GL.GL_LINE_SMOOTH);
	    gl.glEnable(GL.GL_DEPTH_TEST);										// Enables Depth Testing
	    gl.glDepthFunc(GL.GL_LEQUAL);										// The Type Of Depth Test To Do
	    gl.glHint(GL2.GL_LINE_SMOOTH_HINT, GL.GL_DONT_CARE);
	    gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);			// Really Nice Perspective Calculations
	    gl.glEnable(GL.GL_BLEND);
	    gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);

	    m_mouseTracker.reset();
	    m_mouseTracker.setWidthHeight(m_width, m_height);
	  }

	@Override
	  public void reshape(GLAutoDrawable drawable, int x, int y, int w2, int h2) {
	    GL2 gl = drawable.getGL().getGL2();

	    m_width = drawable.getWidth();
	    m_height = drawable.getHeight();
	    m_mouseTracker.setWidthHeight(m_width, m_height);

	    gl.glMatrixMode(GL2.GL_MODELVIEW);
	    loadTrackballTransformation(gl);
	    loadTrackballZoomTranslate(gl);
	    gl.glLoadIdentity();

	    // perspective view
	    gl.glViewport(0, 0, m_width, m_height);
	    gl.glMatrixMode(GL2.GL_PROJECTION);
	    gl.glLoadIdentity();
	    glu.gluPerspective(45.0f,(float)m_width/(float)m_height,0.1f,100.0f);
	  }

	  public void addToShift(int x, int y) {
	    m_xShift += x;
	    m_yShift += y;
	  }
	
	public void drawScene(GLAutoDrawable drawable) {
		GL2 gl = drawable.getGL().getGL2();

	    //angle+=.1f;

	    gl.glMatrixMode(GL2.GL_MODELVIEW);
	    //gl.glLoadIdentity();
	    
	    
	    //loadTrackballTransformation(gl);
	    //loadTrackballZoomTranslate(gl);

	    //	    gl.glTranslated(m_xShift, -m_yShift, 0d);
	    //glu.gluLookAt(m_xShift, -m_yShift, 0d, /* look from camera XYZ */ 
	    //    0, 0, 0, /* look at the origin */ 
	    //    0, 0, 1); /* positive Y up vector */ 

	    // Move back 6 units
	    //gl.glTranslatef( 0.0f, 0.0f,-16.0f );
	    // rotate to top view
	    //gl.glRotatef(90f, 0f, 0f, 0f);

	    // Rotate by angle
	    //gl.glRotatef(angle, angle, 0.0f, 0f);
	    
	    m_bot.drawView(drawable);
	    if(m_isDesOn == true) {
	    	m_desBot.drawView(drawable);
	    }
	    drawGrid(drawable);
	    //drawBot(drawable);
	    //drawDesiredBot(drawable);
	    if(m_isEnvOn == true) {
	    	m_env.drawView(drawable);
	    	if(m_isDesOn == true) {
	    		m_desEnv.drawView(drawable);
	    	}
	    }
	}
	
	private void drawGrid(GLAutoDrawable drawable) {
		GL2 gl = drawable.getGL().getGL2();
		float width = 100;
		float subdivision = 100;
		gl.glPushMatrix();
		gl.glLineWidth(1);
		gl.glColor3f(0.5f, 0.5f, 0.5f);
		gl.glBegin(GL2.GL_LINES);
		gl.glEnable(GL2.GL_LINE_SMOOTH);
	    gl.glHint(GL2.GL_LINE_SMOOTH_HINT, GL2.GL_NICEST);

		float stepsize = width/subdivision;
    
		for(int i = 0; i<=subdivision; i++){
			gl.glVertex3f(-width/2 + i*stepsize,width/2, 0);
			gl.glVertex3f(-width/2 + i*stepsize,-width/2, 0);

			gl.glVertex3f(width/2,-width/2 + i*stepsize, 0);
			gl.glVertex3f(-width/2,-width/2 + i*stepsize, 0);
		}

		gl.glEnd();
		gl.glPopMatrix();
	}
	
	public void switchIsEnvironmentOn(boolean isOn) {
		m_isEnvOn = isOn;
	}
	
	public void switchIsDesiredOn(boolean isOn) {
		m_isDesOn = isOn;
	}
	
	/*public void drawBot(GLAutoDrawable drawable) {
		GL2 gl = drawable.getGL().getGL2();
		for(ModelBody modelBody : model.getModelBodies()) {
			
			gl.glPushMatrix();
			
			ModelObject parent = modelBody.getParentObject();
			
			// If it has a parent, the parent is a transform and should be applied
			while((parent != null) && (parent instanceof ModelTransform )) {
				ModelTransform transform = (ModelTransform) parent;
				gl.glTranslatef((float) transform.getXShift(), (float) transform.getYShift(), (float) transform.getZShift());
				parent = parent.getParentObject();
			}
			
			// Draw the object
			if(modelBody instanceof ModelBox) {
				ModelBox modelBox = (ModelBox) modelBody;
				double w = modelBox.getODEBoxDims().get0();
				double b = modelBox.getODEBoxDims().get1();
				double h = modelBox.getODEBoxDims().get2();
				double x = ((ModelBody) modelBody).getPosition().getxPos();
				double y = ((ModelBody) modelBody).getPosition().getyPos();
				double z = ((ModelBody) modelBody).getPosition().getzPos();
				
				Color c = Color.GREEN;
				if (modelBox.getId().equals("botbody"))
					c = Color.blue;
				
				drawBox(x, y, z, w, b, h, gl, c);
			}
			else if(modelBody instanceof ModelCylinder) {
				double len = ((ModelCylinder) modelBody).getLength();
				double rad = ((ModelCylinder) modelBody).getRadius();
				double x = ((ModelBody) modelBody).getPosition().getxPos();
				double y = ((ModelBody) modelBody).getPosition().getyPos();
				double z = ((ModelBody) modelBody).getPosition().getzPos();
				
				drawCylinder(x, y, z, len, rad, gl);
			}
			// TODO spheres
			
			gl.glPopMatrix();
		}
	}
	
	public void drawDesiredBot(GLAutoDrawable drawable) {
		Collection<ModelBody> dbodies = desiredView.getDesiredBodies();
		GL2 gl = drawable.getGL().getGL2();
		for(ModelBody modelBody : dbodies) {
			
			gl.glPushMatrix();
			
			ModelObject parent = modelBody.getParentObject();
			
			// If it has a parent, the parent is a transform and should be applied
			while((parent != null) && (parent instanceof ModelTransform )) {
				ModelTransform transform = (ModelTransform) parent;
				gl.glTranslatef((float) transform.getXShift(), (float) transform.getYShift(), (float) transform.getZShift());
				parent = parent.getParentObject();
			}
			
			// Draw the object
			if(modelBody instanceof ModelBox) {
				ModelBox modelBox = (ModelBox) modelBody;
				double w = modelBox.getODEBoxDims().get0();
				double b = modelBox.getODEBoxDims().get1();
				double h = modelBox.getODEBoxDims().get2();
				double x = ((ModelBody) modelBody).getPosition().getxPos();
				double y = ((ModelBody) modelBody).getPosition().getyPos();
				double z = ((ModelBody) modelBody).getPosition().getzPos();
				
				Color c = new Color(1f, 1f, 0f, 0.5f);
				if (modelBox.getId().equals("botbody"))
					c = new Color(1f, 0f, 1f, 0.5f);
				
				drawBox(x, y, z, w, b, h, gl, c);
			}
			else if(modelBody instanceof ModelCylinder) {
				double len = ((ModelCylinder) modelBody).getLength();
				double rad = ((ModelCylinder) modelBody).getRadius();
				double x = ((ModelBody) modelBody).getPosition().getxPos();
				double y = ((ModelBody) modelBody).getPosition().getyPos();
				double z = ((ModelBody) modelBody).getPosition().getzPos();
				
				drawCylinder(x, y, z, len, rad, gl);
			}
			// TODO spheres
			
			gl.glPopMatrix();
		}
	}*/
	
	/*private void drawBox(double dx, double dy, double dz, double dw, double db, double dh, GL2 gl, Color c) {
		// need floats for OpenGL
		float x = (float) (dx - (dw /2 ));
		float y = (float) (dy - (db / 2));
		float z = (float) (dz - (dh / 2));
		float w = (float) dw;
		float b = (float) db;
		float h = (float) dh;
		
		gl.glPushMatrix();
	    gl.glBegin(GL.GL_TRIANGLES); // Begin drawing triangle sides
	    
	    // Front panel
	    float[] components = c.getComponents(null);
	    //gl.glColor3i(0, 1, 0);
	    gl.glColor4f(components[0], components[1], components[2], components[3]); //( 0, 0.5f, 1 );
	    gl.glVertex3f( x, y, z ); // Origin
	    gl.glVertex3f(x+w, y, z);
	    gl.glVertex3f( x, y+b, z );
	    
	    //gl.glColor3f( 1, 0, 0 );
	    gl.glVertex3f( x+w, y, z );
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x+w, y+b, z );
	    
	    // Back panel
	    //gl.glColor3f( 0, 1, 0 );
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x+w, y, z+h );
	    gl.glVertex3f( x, y+b, z+h );
	    
	    //gl.glColor3f( 0, 1, 0 );
	    gl.glVertex3f( x+w, y, z+h );
	    gl.glVertex3f( x+w, y+b, z+h );
	    gl.glVertex3f( x, y+b, z+h );
	    
	    // Left panel
	    //gl.glColor3f( 0, 0, 1 );
	    gl.glVertex3f( x, y+b, z+h );
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x, y, z+h );
	    
	    //gl.glColor3f( 0, 0, 1 );
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x, y, z );
	    
	    // Right panel
	    //gl.glColor3f( 1, 1, 0 );
	    gl.glVertex3f( x+w, y+b, z+h );
	    gl.glVertex3f( x+w, y+b, z );
	    gl.glVertex3f( x+w, y, z+h );
	    
	    //gl.glColor3f( 1, 1, 0 );
	    gl.glVertex3f( x+w, y+b, z );
	    gl.glVertex3f( x+w, y, z+h );
	    gl.glVertex3f( x+w, y, z );
	    
	    // Bottom panel
	    //gl.glColor3f( 1, 0, 1 );
	    gl.glVertex3f( x, y, z );
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x+w, y, z );
	    
	    //gl.glColor3f( 1, 0, 1 );
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x+w, y, z );
	    gl.glVertex3f( x+w, y, z+h );
	    
	    // Top panel
	    //gl.glColor3f( 0, 1, 1 );
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x, y+b, z+h );
	    gl.glVertex3f( x+w, y+b, z );
	    
	    //gl.glColor3f( 0, 1, 1 );
	    gl.glVertex3f( x, y+b, z+h );
	    gl.glVertex3f( x+w, y+b, z );
	    gl.glVertex3f( x+w, y+b, z+h );
	    
	    gl.glEnd();
	    //gl.glFlush();
	    gl.glPopMatrix();
	}
	
	private void drawCylinder(double dx, double dy, double dz, double dlength, double dradius, GL2 gl) {
		float x = (float) dx;
		float y = (float) dy;
		float z = (float) dz;
		float length = (float) dlength;
		float radius = (float) dradius;
		
		
	}*/

}
