/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.view;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.bbn.morph.model.DesiredPhysicalModel;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.ModelBody;
import com.bbn.morph.model.ModelJoint;
import com.bbn.morph.model.PhysicalModel;
import com.bbn.morph.model.RobotAttributeValues;

public class DesiredView {
	  private PhysicalModel m_robot;
	  private DesiredPhysicalModel m_desired;
	  
	  public DesiredView(PhysicalModel robot, DesiredPhysicalModel desired) {
		 m_robot = robot;
		 m_desired = desired;
	  }
	  
	  public void refresh() {
		  
	  }
	  
	  public Set<ModelBody> getDesiredBodies() {
	    Set<RobotAttributeValues> rav = m_desired.getDesiredChanges();

	    Map<String, ModelBody> idMap = new HashMap<String, ModelBody>();
	    Set<ModelBody> desiredBodies = new HashSet<ModelBody>();
		  
		  for(ModelBody modelBody : m_robot.getModelBodies()) {
		    idMap.put(modelBody.getId(), modelBody.clone());
		  }
		  
		  for(RobotAttributeValues r : rav) {
		    String att = r.getAttribute();
		    String id = RobotAttributeValues.getPrefix(att);
		    String postfix = RobotAttributeValues.getPostfix(att);

		    ModelBody modelBody = idMap.get(id);
		    List<ModelJoint> joints = m_robot.getJoints(modelBody);
		    
		    // compute the delta instead of the actual desired value:
		    Measurement delta = r.getMeasurement().subtract(modelBody.getAttributeValue(postfix));
		    
	      modelBody.updateRobotAttributeFromDeltas(new RobotAttributeValues(postfix, delta), joints);
	      
	      desiredBodies.add(modelBody);
		  }

		  return desiredBodies;
	  }

}
