/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.view;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.executer.Update;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.Units;


/**
 * Functional BluePrint implementation. 
 *  
 * @author fyaman
 *
 */
public class FB {
Feature f;
Map<RobotAttribute, UpdateFunction> updateFunctions;
StressFunction stressF;
  private boolean m_isUserInduced = false;


public FB(Feature f, Map<RobotAttribute, UpdateFunction> updateFunctions,
		StressFunction stressF) {
	super();
	this.f = f;
	this.updateFunctions = updateFunctions;
	this.stressF = stressF;
}

public Feature getFeature(){
	return f;
}
public Double stress(StressInputs inputs){
	return stressF.computeStress(inputs); 
}
public Set<Update> updates(Double stress){
	Set<Update> deltas= new HashSet<Update>();
	for(RobotAttribute a: updateFunctions.keySet()){
	   Measurement val= new Measurement(Units.getNone(), updateFunctions.get(a).delta(stress));
	   deltas.add(new Update(a, val));
	}
	return deltas;	
}

@Override
public String toString() {
	StringBuffer buf= new StringBuffer();
		buf. append("FB [f=" + f + "]");//, stressF=" + stressF.description() + ", updateFunctions= {");
		for(RobotAttribute a: updateFunctions.keySet())
			buf.append("(").append(a).append(", ").append(updateFunctions.get(a).description()).append(") ; ");
		buf.append("} ]");
		return buf.toString();
}

/**
 * @return true iff the {@link FB} is user induced
 */
public boolean isUserInduced() {
  return m_isUserInduced;
}

/**
 * @param isUserInduced true means that the {@link FB} is user induced
 */
public void setUserInduced(boolean isUserInduced) {
  m_isUserInduced = isUserInduced;
}





}
