/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.view;

import java.io.File;
import java.util.List;

import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.ModelObject;
import com.bbn.morph.model.ModelUtils;
import com.bbn.morph.model.PhysicalModel;

/**
 * A view that combines the robot and environment into one view
 * @author aadler
 *
 */
public class ProtoModelView extends SimulatorModelView {


  /**
   * Construct a new {@link ProtoModelView}
   * @param robot the robot model
   * @param environment the environment model
   */
  public ProtoModelView(PhysicalModel robot, PhysicalModel environment) {
    super(robot, environment);
  }

  /**
   * Construct a new {@link ProtoModelView} with a step of a certain height.
   * Useful for testing
   * @param robot the robot model
   * @param environment the environment model
   * @param stepHeight the height of the step to make
   */
  public ProtoModelView(PhysicalModel robot, PhysicalModel environment, Measurement stepHeight) {
    super(robot, environment, stepHeight);
  }
  
//  /* (non-Javadoc)
//   * @see com.bbn.morph.view.SimulatorModelView#setVariables()
//   */
//  protected void setVariables() {
//    // wheel constants:
//    m_wheelDistanceFromDriveEdge = new Measurement(Units.getMeters(), 0.021082);
//    m_wheelDistanceFromPassiveEdge = new Measurement(Units.getMeters(), 0.019050);
//    //private static double WHEEL_RADIUS = 0.75;
//    m_wheelLength = new Measurement(Units.getMeters(), 0.0254);
//    m_wheelGap = new Measurement(Units.getMeters(), 0.0025); // gap between wheel and body
//
//    // FIXME the rounded corners really mean this should be negative or something. We need a long term fix
//    m_wheelOverlap = new Measurement(Units.getMeters(), 0.001); // amount of wheel hanging over the corner
//    m_wheelMass = new Measurement(Units.getGrams(), 1.0);
//    
//    m_flipperGap = new Measurement(Units.getMeters(), 0.0); // gap between wheel and flipper
//    
//    m_stepCenterDelta = new Measurement(Units.getMeters(), 1.0);
//  }


  /* (non-Javadoc)
   * @see com.bbn.morph.view.SimulatorModelView#writeToFile(java.io.File)
   */
  // writes file as XML
  public void writeToFile(File file) throws Exception {
    List<ModelObject> objects = m_modifiedRobot.getTopModelObjects();
    List<ModelObject> envObjects = m_environment.getTopModelObjects();
    objects.addAll(envObjects);
    ModelUtils.writeToXMLFile(file, objects);
  }
  
}
