/*
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.view;

import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.PhysicalModel;


/** 
 *  This is the update function that might be useful for desired models
 *  It computes a delta that is either a constant factor of the current value
 *  or the target value whichever is smaller.
 *   
 * @author fyaman
 *
 */
public class RelativeIncrementDecrementUpdate implements UpdateFunction {

double updatePercentage=0;
private PhysicalModel m_model;
private RobotAttribute m_attribute;
private Measurement m_targetValue;
private int sign=1; 


public RelativeIncrementDecrementUpdate(double step, PhysicalModel m, RobotAttribute a, Measurement targetValue) {
	this(step, m,a, targetValue,false);
}

public RelativeIncrementDecrementUpdate(double step, PhysicalModel m, RobotAttribute a, Measurement finalValue, boolean isReverse) {
	this.m_model=m;
	this.m_attribute = a;
	this.m_targetValue = finalValue;
	this.updatePercentage = step;
	if (isReverse)
		sign=-1;
}

@Override
public double delta(double stress) {
	if (stress==0 )
		return 0;
	Measurement currentVal = getCurrentValue();
	double diff = m_targetValue.getStandardValue() - currentVal.getStandardValue();
	double delta = Math.min(currentVal.getStandardValue()*updatePercentage, Math.abs(diff));
	if (stress < 0 )
		return -delta * sign;
	return delta*sign;
}

private Measurement getCurrentValue() {
	AttributeValueView view = m_model.getAttributeValueView();
	return view.get(m_attribute);
}

@Override
public String description() {
	return "RelativeIncrement Update Function "+updatePercentage+"% update with target " + m_targetValue;
}

}
