# XML-RPC server for auto-generating ROS config files and kicking off simulations
 
# Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
# in the AUTHORS file in the MADV distribution's top directory.

# This file is part of MADV, and is distributed under the terms of the
# GNU General Public License, with a linking exception, as described in
# the file LICENSE in the MADV distribution's top directory. 

from twisted.web import xmlrpc, server
from subprocess import call
import time
import os
import subprocess
import multiprocessing
from threading import Thread

def returnStress(d):
        """
        Data handling function to be added as a callback: handles the
        data by printing the result
        """
        return d

def launchRos():
    subprocess.call('/opt/ros/diamondback/ros/bin/roslaunch $MADV_ROS_PKG/madv.launch', shell=True)


def runSim(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent):
        print("Writing out ROS configuration files for client " + client)
        
        #path = '../../../java/resources/xmlModels/'
        path = '$MADV_ROS_PKG/'
        f = open(path+'dimension.urdf.xacro', 'w')
        f.write(dimensionsContent);
        f.close()
        
        f = open(path+'wheels.urdf.xacro', 'w')
        f.write(wheelsContent);
        f.close()
        
        f = open(path+'madv_wheel_controller.yaml', 'w')
        f.write(wheelControllerContent);
        f.close()
        
        f = open(path+'controllers.launch', 'w')
        f.write(controllersContent);
        f.close()
        
        f = open(path+'stair.model', 'w')
        f.write(stairContent);
        f.close()
        
        f = open(path+'wheels.py', 'w')
        f.write(numWheelsContent);
        f.close()
        
                
        f = open(path+'bot_spawn.launch', 'w')
        f.write(botSpawnContent);
        f.close()
        
        print("Wrote configuration files")
        Example.processes[client] = multiprocessing.Process(target = launchRos)
        Example.processes[client].start()
        Example.processes[client].join()
        value = Example.values[client]
        Example.deferreds[client].callback(value)
        return

def sendValue(self, rosThread, d, value):
    rosThread.join()
    d.callback(float(value))
    return



class Example(xmlrpc.XMLRPC):
    """
    An example object to be published.
    """
    deferreds = {}
    processes = {}
    experiments = {}
    values = {}

        
    def xmlrpc_evaluate(self, client, value):
        Example.values[client] = value
        return 'DONE'

    def xmlrpc_request2(self, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent):
        client = 'madv'
        d = defer.Deferred()
        d.addCallback(returnStress)
        Example.deferreds[client] = d
        sim=Thread(target = runSim, args=(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent))
        #sim = multiprocessing.Process(target = runSim, args=(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent))
        sim.start()
        #p.join()
        return d

if __name__ == '__main__':
    from twisted.internet import *
    r = Example()
    print( "Starting twistd reactor" )
    reactor.listenTCP(7080, server.Site(r))
    reactor.run()
